@@ -7,8 +7,11 @@
 #ifndef KOSMINDOORMAP_MAPCSSSELECTOR_P_H
 #define KOSMINDOORMAP_MAPCSSSELECTOR_P_H
 
+#include "mapcsstypes.h"
+
 #include <osm/datatypes.h>
 
+#include <functional>
 #include <memory>
 #include <vector>
 
@@ -30,9 +33,11 @@ public:
     /** Resolve tag keys. */
     virtual void compile(const OSM::DataSet &dataSet) = 0;
     /** Returns @c true if this selector matches the evaluation state. */
-    virtual bool matches(const MapCSSState &state, const MapCSSResult &result) const = 0;
+    virtual bool matches(const MapCSSState &state, MapCSSResult &result, const std::function<void(MapCSSResult&, LayerSelectorKey)> &matchCallback) const = 0;
     /** Selector matches the canvas element. */
     virtual bool matchesCanvas(const MapCSSState &state) const = 0;
+    /** The layer selector of this style selector (invalid for union selectors). */
+    virtual LayerSelectorKey layerSelector() const = 0;
 
     virtual void write(QIODevice *out) const = 0;
 
@@ -59,20 +64,23 @@ public:
     ObjectType objectType;
 
     void compile(const OSM::DataSet &dataSet) override;
-    bool matches(const MapCSSState &state, const MapCSSResult &result) const override;
+    bool matches(const MapCSSState &state, MapCSSResult &result, const std::function<void(MapCSSResult&, LayerSelectorKey)> &matchCallback) const override;
     bool matchesCanvas(const MapCSSState &state) const override;
+    LayerSelectorKey layerSelector() const override;
     void write(QIODevice* out) const override;
 
     /** @internal only to be used by the parser */
     void setObjectType(const char *str, std::size_t len);
     void setZoomRange(int low, int high);
     void setConditions(MapCSSConditionHolder *conds);
-    void setClass(const char *str, std::size_t len);
+    void setClass(ClassSelectorKey key);
+    void setLayer(LayerSelectorKey key);
 
     std::vector<std::unique_ptr<MapCSSCondition>> conditions;
     OSM::TagKey m_areaKey;
     OSM::TagKey m_typeKey;
-    QByteArray m_class;
+    ClassSelectorKey m_class;
+    LayerSelectorKey m_layer;
     int m_zoomLow = 0;
     int m_zoomHigh = 0;
 };
@@ -82,8 +90,9 @@ class MapCSSChainedSelector : public MapCSSSelector
 {
 public:
     void compile(const OSM::DataSet &dataSet) override;
-    bool matches(const MapCSSState &state, const MapCSSResult &result) const override;
+    bool matches(const MapCSSState &state, MapCSSResult &result, const std::function<void(MapCSSResult&, LayerSelectorKey)> &matchCallback) const override;
     bool matchesCanvas(const MapCSSState &state) const override;
+    LayerSelectorKey layerSelector() const override;
     void write(QIODevice* out) const override;
     std::vector<std::unique_ptr<MapCSSBasicSelector>> selectors;
 };
@@ -96,10 +105,20 @@ public:
     ~MapCSSUnionSelector();
 
     void compile(const OSM::DataSet &dataSet) override;
-    bool matches(const MapCSSState &state, const MapCSSResult &result) const override;
+    bool matches(const MapCSSState &state, MapCSSResult &result, const std::function<void(MapCSSResult&, LayerSelectorKey)> &matchCallback) const override;
     bool matchesCanvas(const MapCSSState &state) const override;
+    LayerSelectorKey layerSelector() const override;
     void write(QIODevice* out) const override;
-    std::vector<std::unique_ptr<MapCSSSelector>> selectors;
+
+    /** @internal */
+    void addSelector(std::unique_ptr<MapCSSSelector> &&selector);
+
+private:
+    struct SelectorMap {
+        LayerSelectorKey layer;
+        std::vector<std::unique_ptr<MapCSSSelector>> selectors;
+    };
+    std::vector<SelectorMap> m_selectors;
 };
 
 }
